package cn.com.ty.lift.batch.applet.dao.model;

import cn.com.ty.lift.batch.applet.constants.DataStatisticsConstants;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * @author huangyuan
 * @date 2020/2/3
 * @description
 */
@Data
public class AnnualInspectionDataModel extends BaseDataModel {
    /**
     * 计划年检时间
     */
    private LocalDate planDate;
    /**
     * 年检状态（0 待完成；1 已完成；2 超期）
     */
    private Integer status;
    /**
     * stepStatus  :年检阶段状态
     * 未开始0,第一阶段 1-2,第二阶段 3,第三阶段 4-7,第四阶段 8-10
     * <p>
     * 1  维保工确认年检计划
     * 2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
     * 文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
     * --------------------------------
     * 3  企业文员确认现场检验时间
     * 录入人员信息，政府质检，企业质检，联系电话
     * --------------------------------
     * 4	维保工上传检验结果
     * 5	企业文员确认检验结果-合格
     * 6	企业文员确认检验结果-整改
     * 7	企业文员确认检验结果-不合格
     * --------------------------------
     * 8	维保工上传年检报告和合格证
     * 9	企业文员确认年检报告和合格证-合格
     * 10	企业文员确认年检报告和合格证-不合格
     */
    private Integer stepStatus;
    /**
     * 项目名称
     */
    private String projectName;

    /**
     * @param
     * @return
     * @description 获取年检计划时间，格式为yyyy-MM-dd
     */
    public LocalDate getPlanDate() {
        if (planDate != null) {
            //转换日期格式为yyyy-MM-dd
            String planDateStr = planDate.format(DateTimeFormatter.ofPattern(DataStatisticsConstants.PLAN_DATE_FORMAT));
            if (StringUtils.isNotBlank(planDateStr)) {
                return LocalDate.parse(planDateStr, DateTimeFormatter.ofPattern(DataStatisticsConstants.PLAN_DATE_FORMAT));
            }
        }
        return null;
    }
}
