package cn.com.ty.lift.business.evaluation.dao.entity;

import com.baomidou.mybatisplus.annotation.TableId;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 保养单用户评价
 * </p>
 *
 * @author huangy
 * @since 2019-12-31
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class Evaluation implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId("id")
    private Long id;

    /**
     * 电梯ID
     */
    @Range(min = 1,message = "电梯有误")
    private Long liftId;

    /**
     * 维修/保养记录ID
     */
    @NotNull(message = "维修/保养记录不能为空")
    @Range(min = 1,message = "维修/保养记录有误")
    private Long recordId;

    /**
     * 来源（1 维保；2 急修）
     */
    @Range(min = 1,max = 2,message = "评价来源有误")
    private Integer source;

    /**
     * 类型（1 物业评价；2 手写评价；3 自动评价）
     */
    @NotNull(message = "评价类型不能为空")
    @Range(min = 1,max = 3,message = "评价类型有误")
    private Integer type;

    /**
     * 用户建议
     */
    @NotEmpty(message = "用户建议不能为空")
    private String advice;

    /**
     * 服务态度
     */
    @NotNull(message = "服务态度不能为空")
    @Range(min = 1,max = 5,message = "服务态度只能为1~5")
    private Integer serviceLevel;

    /**
     * 保养满意度
     */
    @NotNull(message = "保养满意度不能为空")
    @Range(min = 1,max = 5,message = "保养满意度只能为1~5")
    private Integer starLevel;

    /**
     * 手写签名图片
     */
    @NotEmpty(message = "手写签名图片不能为空")
    private String imgUrl;

    /**
     * 评价时间
     */
    private LocalDateTime postDate;

    /**
     * 评价人ID
     */
    @Range(min = 1,message = "评价人有误")
    private Long userId;
}
